//
//  TimerWindowController.swift
//  Timer
//
//  Designed by Lech Szymanski in 2014.
//  Swift implementation by David Eyers in 2015.
//  Updated for Swift 4.x / Xcode 9.4 in 2018 by Hamza Bennani.
//

import Cocoa

@objcMembers class TimerWindowController: NSWindowController {

    // outlets for setting preferences
    @IBOutlet weak var timerDisplay: NSTextField!
    @IBOutlet weak var startStopButton: NSButton!
    @IBOutlet weak var resetButton: NSButton!
    
    // Timer state is now in an instance of the model
    @objc dynamic var timer:TimerModel? = nil
    
    // how many windows are open?
    static var windowCounter:Int = 0;
    
    convenience init() {
        self.init(windowNibName: NSNib.Name(rawValue: "TimerWindowController"))
 
        // create an instance of a TimerModel and reset it
        timer = TimerModel()
        
        // Register ourselves for notifications about preference changes
        let nc = NotificationCenter.default
        nc.addObserver(self, selector: #selector(TimerWindowController.backChangeHandler(_:)), name: NSNotification.Name(rawValue: TimerBackColorKey), object: nil)
        nc.addObserver(self, selector: #selector(TimerWindowController.fontChangeHandler(_:)), name: NSNotification.Name(rawValue: TimerFontColorKey), object: nil)
        nc.addObserver(self, selector: #selector(TimerWindowController.buttonsChangeHandler(_:)), name: NSNotification.Name(rawValue: TimerBttnsVisibleKey), object: nil)
        
        timer!.reset()
    }
    
    override func windowDidLoad() {
        super.windowDidLoad()

        // Implement this method to handle any initialization after your window controller's window has been loaded from its nib file.
        TimerWindowController.windowCounter += 1
        window?.title = "Timer \(TimerWindowController.windowCounter)"
        
        // Uncomment the following code to check the responder chain is as you expect
        /*
        let win = self.window
        NSLog("Initial first responder \(win?.initialFirstResponder)")
        NSLog("Content view \(win?.contentView)")
        NSLog("Next responder \(win?.contentView.nextResponder)")
        NSLog("Next, next responder \(win?.contentView.nextResponder!!.nextResponder)")
        */

        // Read preferences from user defaults and set the controls in the window appropriately
        setBackColor(PreferencesController.preferencesBackColor())
        setFontColor(PreferencesController.preferencesFontColor())
        setButtons(PreferencesController.preferencesButtonsVisible())
    }
    
    func windowWillReturnUndoManager(_ w:NSWindow)->UndoManager? {
        return timer?.undomngr
    }
    
    // Toggle the state of the timer
    @IBAction func startStopAction(_ sender:AnyObject) {
        if(timer!.running) {
            timer?.stop()
        } else {
            timer?.start()
        }
    }
    
    // The reset action zeros the counter
    @IBAction func resetAction(_ sender:AnyObject){
        timer!.reset()
    }
    
    // When users interact with a menu, the system checks whether items should be greyed out.
    // We use our "running" state to determine whether to grey out start or stop.
    override func validateMenuItem(_ menuItem: NSMenuItem) -> Bool {
    
        let del = NSApplication.shared.delegate as! AppDelegate

        if(menuItem == del.startMenuItem) {
            return !timer!.running
        } else if (menuItem == del.stopMenuItem) {
            return timer!.running
        }
        return true
    }
    
    @objc func backChangeHandler(_ note:Notification){
        if let u = note.userInfo {
            if let c = u["color"] as? NSColor {
                setBackColor(c)
            }
        }
    }
    @objc  func fontChangeHandler(_ note:Notification){
        if let u = note.userInfo {
            if let c = u["color"] as? NSColor {
                setFontColor(c)
            }
        }
    }
    @objc func buttonsChangeHandler(_ note:Notification){
        if let u = note.userInfo {
            if let b = u["buttons"] as? Bool {
                setButtons(b)
            }
        }
    }
    func setBackColor(_ color:NSColor){
        let view = window?.contentView as! TimerView
        view.backColor = color
        view.needsDisplay = true
    }
    func setFontColor(_ color:NSColor){
        timerDisplay.textColor = color
    }
    func setButtons(_ visible:Bool){
        startStopButton.isTransparent = !visible
        startStopButton.isEnabled = visible
        resetButton.isTransparent = !visible
        resetButton.isEnabled = visible
    }
}
